﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioLab.Common;
using BioLab.ImageProcessing;
using System.Drawing;
using BioLab.GUI.Forms;
using System.Windows.Forms;
using BioLab.ImageProcessing.Topology;
using BioLab.DataStructures;
using System.ComponentModel;
using BioLab.GUI.UserControls;

namespace PRLab.FEI
{

    [AlgorithmInfo("Convoluzione (da immagine byte a immagine int)", Category = "FEI")]
    [CustomAlgorithmPreviewParameterControl(typeof(SimpleConvolutionParameterControl))]
    public class ConvoluzioneByteInt : Convolution<byte, int, ConvolutionFilter<int>>
    {
        public ConvoluzioneByteInt(Image<byte> inputImage, ConvolutionFilter<int> filter)
          : base(inputImage, filter, 0)
        {
        }

        public override void Run()
        {
            Result = new Image<int>(InputImage.Width, InputImage.Height);
            var iterator = new ImageBlockIteratorWithBorder(InputImage, Filter.Size, 1, AdditionalBorder);
            int[] filterOffsets;
            int[] filterValues = Filter.CalculateConvolutionValuesAndOffsets(InputImage.Width, out filterOffsets);

            do
            {
                int val = 0;
                int index = iterator.CurrentBlockCenterIndex;
                for (int i = 0; i < filterValues.Length; i++)
                {
                    val += InputImage[index + filterOffsets[i]] * filterValues[i];
                }
                Result[index] = val / Filter.Denominator;
            } while (iterator.MoveNext());
        }
    }

    [AlgorithmInfo("Smoothing di un'immagine", Category = "FEI")]
    public class ImageSmoothing : ImageOperation<Image<byte>, Image<byte>>
    {
        [AlgorithmParameter]
        public int Size { get; set; }

        public override void Run()
        {
            var filter = new ConvolutionFilter<int>(Size, Size * Size);
            for (int i = 0; i < Size * Size; i++)
            {
                filter[i] = 1;
            }
            Result = new ConvoluzioneByteInt(InputImage, filter).Execute().ToByteImage(PixelConversionMethod.Clip);
        }
    }

    [AlgorithmInfo("Sharpening di un'immagine", Category = "FEI")]
    [CustomAlgorithmPreviewParameterControl(typeof(ImageSharpeningParameterControl))]
    public class ImageSharpening : ImageOperation<Image<byte>, Image<byte>>
    {
        [AlgorithmParameter]
        public double Strength { get; set; }

        public override void Run()
        {
            var filter = new ConvolutionFilter<int>(new int[] { -1, -1, -1,
                                                          -1,  8, -1,
                                                          -1, -1, -1 }, 1);
            var tmp = new ConvoluzioneByteInt(InputImage, filter).Execute();

            Result = InputImage.Clone();
            for (int i = 0; i < tmp.PixelCount; i++)
            {
                Result[i] = (Result[i] + Strength * tmp[i]).RoundAndClipToByte();
            }
        }
    }


}
