﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BioLab.GUI.Forms;
using BioLab.ImageProcessing;
using PRLab.FEI;

namespace BioLab.GUI.UserControls
{
    internal partial class ImageSharpeningParameterControl : UserControl, IAlgorithmPreviewParameters
    {
        public ImageSharpeningParameterControl()
        {
            InitializeComponent();
            UpdateLabel();
        }

        const double maxStrength = 5;

        public void ExchangeValues(BioLab.Common.IAlgorithm algorithm, bool toControl)
        {
            var op = (ImageSharpening)algorithm;
            if (toControl)
            {
                trackBarStrength.Value = (int)System.Math.Round(op.Strength * 100 / maxStrength);
            }
            else
            {
                op.Strength = GetStrength();
            }
        }

        protected virtual void OnParametersChanged()
        {
            EventHandler tmp = ParametersChanged;
            if (tmp != null)
                tmp(this, EventArgs.Empty);
        }

        public event EventHandler ParametersChanged;

        private void trackBarStrength_ValueChanged(object sender, EventArgs e)
        {
            UpdateLabel();
            OnParametersChanged();
        }

        private void UpdateLabel()
        {
            labelValue.Text = string.Format("{0:0.0}", GetStrength());
        }

        private double GetStrength()
        {
            return trackBarStrength.Value * maxStrength / trackBarStrength.Maximum;
        }        
    }
}
